;**** A P P L I C A T I O N   N O T E   LCD-Anzeige  ************************
;*
;* Title:		Ansteuerung LCD ber Ser.Schnittstelle
;*                      Display Modell 2
;* Version:		0.93a
;* Last updated:	02.09.2003
;* Target:		AT90S4433
;* 			kleinere Typen nicht mehr mglich bzw. 
;*                      man muss SETUP ausklammern und EEPROM anpassen
;* Author:              Majunke Michael
;* Support E-mail:	majunke@t-online.de
;* Home-Page            http://majunke.onlinehome.de
;*
;* Beschreibung:
;* mit diesem Programm kann ein 16*4/20x4 (1-63)/(1-4) o.a. LCD-Display angesteuert werden
;* das mit einen HD44780 Kompatiblen Controller arbeitet.
;* die Zeichen die an die Ser. Schnittstelle gesendet werden, gibt das Display
;* aus und reagiert auf Befehle.. s. Anlage..
;* weiterhin kann das Display Daten an den Rechner senden..
;* Standart Einstellungen:
;*	- 38400 Baud 		- vernderbar
;*	- Flusskontrolle CTS	- vernderbar
;* 	- Beleuchtung AUTO	- vernderbar
;*
;*	- SETUP -> Taster 2 beim Start ODER
;*	-	-> Taster 1 UND 2 gleichzeitig
;*
;* nderungen:
;* Es gab noch einen Fehler in der Neuen Locate-Funktion bei Displays
;* ab 20x4 - den habe ich behoben.
;* 
;* Neue LocateZ_S Methode : es knnen jetzt 0-3Zeilen und 
;*                          0-63Spalten angesteuert werden
;*                          Codierung s. Methode oder iodefs.inc
;*                        : die berprffung auf gltige ~P Eingaben wird dabei
;*                          jeweils auf die aktuelle Konfiguration bezogen
;*                        : Achtunf, fr Displays grer 16x4 konnte ich nicht 
;*                          alles Vollstndig austeste
;* CTS am Anfrang gelscht.. lst evt. problem, das Display manchmal nicht auf Bereit geht
;*
;*
;* geplant:
;*    Treiber fr andere Display-Controller
;*
;* freien AD-WANDLER-Eingang integrieren (anzeige oder ser. Ausgabe Messwert)
;* evt. Erweiterungen des Menus
;* Codeoptimierungen
;* 
;* GPL
;*****************************************************************************


;*****************************************************************************
;*
;* Displaytypen Nr.: = Displaytyp
;* 		
;*		1 = 16x4 HD44780 Kompatible
;*		2 = 20x4 HD44780 Kompatible
;*              3 = 40x2 HD44780 Kompatible
;*              4 = 40x4 HD44780 Kompatible
;*              5 =  
;*
;* avra --define DISPLAY_TYPE=DisplayTypNr
;*****************************************************************************

;*** der Standartdisplay Type = 1
.ifndef	DISPLAY_TYPE
	.define	DISPLAY_TYPE	1
.endif



.DEVICE	AT90S4433 

; *** Definitionen der Register, Variablen, Speicher
.INCLUDE	"DM2-iodefs.inc"


; ******************************************************************************
;*** Sprunktabelle fr Interrups ***
.CSEG

.ORG	$0
Reset:			rjmp	Init_Reset	; Reset behandlung, Neuinitaliesieren
IRQ0:			reti			;
IRQ1:			reti			;
TIMER1_CAP:		reti			;
TIMER1_COMP:		reti			;
TIMER1_OVFL:		rjmp	TIMER1_OL	; Timer 16bit, Helligsteuerung, LED blinken
TIMER0_OVFL:		rjmp    TIMER0_OL	; Timer 8bit zur Tasterabfrage
SPI_TRANSF_COMPL:	reti			;
UART_RX_COMPL:		rjmp	UART_RX_C	; ein zeichen wurde empfangen
UART_DR_EMPTY:		reti			;
UART_TX_COMPL:		reti			;
ADC_CONV_COMPL:		rjmp  	ADC_CONV_C	; eine AD-wandlung ist fertig
EE_RDY:			reti			;
ANA_COMP:		reti			;


; Texte (aus dem EEPROM genohmen da EEPROM Speichestellen nicht stabil)
; immer GERADE ANZAHL von Zeichen !!! max. 16 Zeichen

Text_AN:                .DB     "AN", 0			; 2
Text_AD:                .DB     "AD", 0

Text_EXIT:              .DB     "EXIT", 0		; 4
Text_NEXT:              .DB     "NEXT", 0
Text_BACK:		.DB	"BACK", 0
Text_UART:              .DB     "UART", 0
Text_BEL:               .DB     "BEL ", 0
Text_AUS:               .DB     "AUS ", 0
Text_AUTO:              .DB     "AUTO", 0
Text_BAUD:              .DB     "BAUD", 0
Text_CTS:               .DB     "CTS ", 0
Text_14400:		.DB	"14k4", 0
Text_19200:		.DB	"19k2", 0
Text_38400:		.DB	"38k4", 0

TEXT_iV:		.DB	"FOLGT ", 0		; 6
Text_2400:		.DB	"2400k ", 0
Text_9600:		.DB	"9600k ", 0
Text_FLUSS:             .DB     "FLUSS ", 0
Text_TIMER:             .DB     "TIMER ", 0
Text_Modus:		.DB	"MODUS ", 0
Text_SETUP:		.DB	"SETUP ", 0
Text_INFO:              .DB     "STATUS", 0
Text_TAST:              .DB     "TASTER", 0

Text_GESETZT:		.DB	"GESETZT ", 0		; 8

Text_SOFT:              .DB     "SOFTWARE  ", 0		; 10
Text_PRELLZEIT:         .DB     "PRELLZEIT ", 0
Text_Hell:		.DB	"HELLIGKEIT", 0
Text_Schwelle:		.DB	"SCHALTWERT", 0

Text_REP:		.DB	"REPARIERE..WERTE", 0	; 16
Text_FEHLER:		.DB	".FEHLER..EEPROM.", 0

Intro_Meldung_1:	.DB	"..Display Init..", 0
Intro_Meldung_2:	.DB	".Software V0.93a", 0	


;****** Routine nach Reset bzw. Neustart - Vorinitalisieren
Init_Reset:
	cli				; Interupts aus
	;*** Initialize stack pointer
	;* StackPointer auf hchste Adresse im SRAM legen
	ldi	WORK, RAMEND	 	; SP auf RAM_END_Adresse 
	out	SP, WORK		; und setzen
	
	;*** I/O-Pins in Schaltung auf defin. Werte einstellen
	; Port D 
	ldi	WORK,0b00111110  	; Rxd/TxD/cts/Led1-3/taster1-2 Richtung s. tabelle 
	out	PORTD_DDR,WORK		; AusgabeRichtung - DataDIR Port D
	ldi	WORK,0b11111000  	; LED/CTS aus .. Taster pull-up
	out	PORTD_D,WORK		; Daten - Data Port D
	; Port C
	ldi	WORK,0b00000011	 	; E/Bel./taster3/helligkeit/taster4/ad Richtung
	out	PORTC_DDR,WORK		; Ausgaberichtung - DataDIR Port C
	ldi	WORK,0b00010101		; Bel. aus / E high
	out	PORTC_D,WORK		; Daten - DataDIR Port C

	; testen ob Beleuchtung an oder aus sein soll	
	ldi	WORK, Bel_ONorOFF	; wenn Nicht AUTOM. dan Belecht on oder AUS aus EEPROM lesen
	out 	EEADRESS,  WORK		; setze EEPROM-Adresse auf Adresse
	sbi	EECONTR,0		; setze das Daten gelsene werden sollen
	in	WORK, EEDAT		; und schiebe die Daten nach WORK
	tst	WORK
	breq	Bel_Init_OFF
	sbi	PORTC_D, 1
	Bel_Init_OFF:			; als Standart
	
	; Port B
	ldi	WORK,DDR_READ_DISPLAY  	; Datenricht. zum lesen LESEN v. Display
	out	PORTB_DDR,WORK		; Ausgaberichtung - DataDIR Port B
	ldi	WORK,0b111111	  	;
	out	PORTB_D,WORK		; Daten - Data Port B	
	
	; UART - Einstellen 
	ldi	WORK, AKT_UART_TEILER	; Baudrate aus EEPROM lesen
	rcall	Eeprom_Pruef_Werte	; suche nach 2 gleichen Werten 
;out 	EEADRESS,  WORK		; setze EEPROM-Adresse auf Adresse
;sbi	EECONTR,0		; setze das Daten gelsene werden sollen
;in	WORK, EEDAT		; und schiebe die Daten nach WORK
	out	UART_UBRR_L,WORK	; low-teil
	clr	WORK
	out	UART_UBRR_H,WORK	; h-teil

	ldi	WORK,UART_CR_OFF	; Betriebsart des UART auf AUS
	out	UART_UCSRB,WORK
	; erst spter anschalten da SETUP

	; Flusskontrolle einstellen XON or CTS 
	; aus EEPROM UART_FLUSS lesen und in Register UART_FLUSS_REG zwischenspeichern s.o.
	ldi	WORK, UART_FLUSS
	out 	EEADRESS,  WORK		   ; setze EEPROM-Adresse auf Adresse
	sbi	EECONTR, 0		   ; setze das Daten gelsene werden sollen
	in	WORK, EEDAT
	mov	UART_FLUSS_REG, WORK

	; Puffer Variablen setzen
	ldi	WORK, PUFFER 		
	mov	PUFFER_POS, WORK	; Adresse v. Puffer in Reg. 
	clr	PUFFER_AKT_LAENG	; akt. Gre (zeichenanzahl) lschen
	clr	C_MERK			; Akt. CursorPosition lschen
	
	; Taster und Timerpuffer
	clr 	TASTER_PRESSED          ; kein Taster gedrckt

	; Tasterentprellzeiten einstellen
	ldi	WORK, TASTER_T_Counter_Time ; Enprellzeit aus EEPROM lesen
	rcall	Eeprom_Pruef_Werte	; suche nach 2 gleichen Werten 
;out 	EEADRESS,  WORK		   ; setze EEPROM-Adresse auf Adresse
;sbi	EECONTR,0		   ; setze das Daten gelsene werden sollen
;in	WORK, EEDAT		   ; und schiebe die Daten nach WORK
	mov	TIMER_T1_COUNTER, WORK     
	mov	TIMER_T2_COUNTER, WORK
	mov	TIMER_T3_COUNTER, WORK
	mov	TIMER_T4_COUNTER, WORK
	
		
	; Timer1 Standart ANSCHALTEN
	clr	WORK			; Reg 1A einstellen  betriebsart 16bit Counter 
	out	TCCR1A, WORK		; 
	ldi	WORK,Timer_Clock_1	; Reg 1B einstellen  Frequenz = Einstellung oben
	out	TCCR1B, WORK
	ldi	WORK, Timer1_Set	; Timer1 OverflowInterupt AN
	out	TIMSK, WORK
	
	; Timer0 Standart ANSCHALTEN
	ldi	WORK,Timer_Clock_0	; TCCR0 einstellen  Frequenz = s. Einstellung oben
	out	TCCR0, WORK
	in	WORK, TIMSK		; Timer 0 AN
 	ori	WORK, Timer0_Set
	out	TIMSK, WORK

	; Standarteinstell. Welche Routinen sollen in den Timern durchlaufen werden
	ldi	WORK, Timer_Aktiv_Byte_Def ; timer start init aus EEPROM lesen
	rcall	Eeprom_Pruef_Werte	; suche nach 2 gleichen Werten 
;out 	EEADRESS,  WORK		   ; setze EEPROM-Adresse auf Adresse
;sbi	EECONTR,0		   ; setze das Daten gelsene werden sollen
;in	WORK, EEDAT		   ; und schiebe die Daten nach WORK
	mov	TIMER_AKTIV_BYTE, WORK		

	ldi	WORK, 0xff		
	mov	TIMER_TAKT, WORK	 ; Timertakt setzen (zhlt alle 0.5sec -1)
	
	; Die Beleuchtungshystereschwelle in Register ablegen
	ldi	WORK, Helligk_Schwelle_Min 	; AKT. Einlesen
	out 	EEADRESS, WORK			; setze EEPROM-Adresse auf Adresse
	sbi	EECONTR,0			; setze das Daten gelsene werden sollen
	in	REG_HELLIGK_SCHWELLE_MIN, EEDAT	; und schiebe die Daten nach WORK
	
	
; ******************************************************************************	
; **** Main ********************************************************************
; ****                                                                     *****
Start:
	rcall	Init_Display		;*** Initalisieren des Display
	rcall	Intro			;*** Intro anzeigen

	rcall	Eeprom_Pruef		; *** Extra EEPROM Prfen MIT Fehlerbehebung !
	
	sei				; schalte IRQs zu
	rcall	Setup_MAIN		;*** nach auswahl ins SETUP
	ldi	WORK,UART_CR		; UART anschalten -> Ready zum Zeichenempfang
	out	UART_UCSRB,WORK

	tst	UART_FLUSS_REG		; wird CTS oder XON verwendet ?
	breq	NO_SOFTWARE_FLUSS_1
	ldi 	WORK,17			; sende ein XON (asc17) - Display bereit zum 
    	out 	UART_UDR,WORK		; Zeichenempfang
	NO_SOFTWARE_FLUSS_1:
	sbi	PORTD_DDR,2		; cts aus
	cbi	PORTD_D,2		; damit neue Zeichnen kommen knnen


  Start_SerIn:				;*** starte Hauptschleife
	rcall   Taster_Abfrage		; Abfrage nach gedrckten Tasten
	tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ?
	breq	Start_SerIn		; ja 
	rcall	Read_Puffer		; dann lese das Zeichen aus Puffer
   Code_Empfang:			; und Werte es aus ...
	cpi	DISPLAY_DAT,ASC_CODE	; ist es das Code-Einleitungs-Zeichen ?
	breq	Code_Locate_Cursor	; ja, dann CODE-Behandlung
	rjmp 	Main_Send_Data_MIT_LOC	; sonst ausgeben ALS NORMALER TEXT
	; *** Befehl Locate Z_S ****** 
        Code_Locate_Cursor:
         tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ? Befehlszeichen holen
	 breq	Code_Locate_Cursor	; nein ? dann warte -> sonst auswerten
	 rcall	Read_Puffer		; *****************	
         clr	CODE_MERK
	 cpi	DISPLAY_DAT,ASC_CODE_LOCATE ; ist es das Code-Zeichen f. Locate ?
	 brne	Code_Clear_Display	; nein, -> test next  ->else Code
	 ; Eigentliche Code fr Locate Z_S
	 Code_Locate_Z:
	 tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ? Zeile
	 breq	Code_Locate_Z		; nein ? dann warte
	 rcall	Read_Puffer		; sonst lesen und entsprechend auswerten
	 subi	DISPLAY_DAT,48		; aus ASC Zahl machen - ZEILE
	 cpi	DISPLAY_DAT, DISPLAY_ZEILEN_ANZAHL		; ber DISPLAY_ZEILEN_ANZAHL-1 ??
	 brlo	Code_Locate_W1		; auerhalb dann ABBRUCH,keine ZEICHENAUSGABE
	 rjmp	End_SerIn		;           (rjmp weil sprungweite zu kurz)
	 Code_Locate_W1:
	 swap	DISPLAY_DAT             ; Zeile Zwischenspeichern und weier Spalte ermitteln
	 lsl	DISPLAY_DAT
	 lsl	DISPLAY_DAT
	 mov	TEMP, DISPLAY_DAT	; Zeile an richtige Position f. AusgabeFunktion 
	 Code_Locate_S:			; 2 Zahlen mssen eingelesen werden fr Spalte !!
	 tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ?
	 breq	Code_Locate_S		; nein ? dann warte
	 rcall	Read_Puffer		; sonst lesen und entsprechend auswerten
	 subi	DISPLAY_DAT,48		; aus ASC Zahl machen - SPALTE*(10hoch1)
	 cpi	DISPLAY_DAT, ( DISPLAY_SPALTEN_ANZAHL / 10 ) + 1		; ber 1 ?? Spalte bis max -> XX Zeichen
	 brlo	Code_Locate_W2		; auerhalb dann ABBRUCH,keine ZEICHENAUSGABE
	 rjmp	End_SerIn		;            ( rjmp weil sprungweite zu kurz)
	 Code_Locate_W2:
	 tst	DISPLAY_DAT
	 breq	Code_Locate_S2		; Null dann direkt 2. Wert lesen,

	 ; ori	CODE_MERK, 10 		; sonst 10hoch1 dazu
	 ; --> NEU grer als nur 1* 10hoch fr 20-> Displays
	 
	 ; --> TODO Testen Displays Groeer 20 Spalten	 	 
	 push	TEMP
	 ldi	TEMP, 10	        ; DISPLAY_DAT * 10
	 
	 MUL_MAL_10:
	 dec	DISPLAY_DAT		; wenn 1 , dann ist TEMP schon 10, Ende
	 breq	END_MUL_MAL_10	 	 
	 ldi	WORK, 10		; sonst nochmal plus 10
	 add	TEMP, WORK	
	 rjmp	MUL_MAL_10
	 
	 END_MUL_MAL_10:	 	 	
	 
	 or 	CODE_MERK, TEMP		; merken	 
	 pop	TEMP
	 
	 ; mit Schleife DISPLAY_DAT 10x +1 ,dies DISPLAY_DAT mal wiederholen  
	 ; ori CODE_MERK, 20 usw...	 
	 
	 
 	 Code_Locate_S2:
 	 tst	PUFFER_AKT_LAENG	; und 10hoch0 lesen
	 breq	Code_Locate_S2		; nein ? dann warte
	 rcall	Read_Puffer		; sonst lesen und entsprechend auswerten
	 subi	DISPLAY_DAT, 48		; aus ASC Zahl machen - SPALTE*(10hoch0)
	 cpi	DISPLAY_DAT, 10		; 0-9 ?  
	 brlo	Code_Locate_W3		; auerhalb dann ABBRUCH,keine ZEICHENAUSGABE
	 rjmp	End_SerIn		;             (rjmp weil sprungweite zu kurz)
	 Code_Locate_W3:
	 add	DISPLAY_DAT, CODE_MERK
	 
	 cpi	DISPLAY_DAT,  DISPLAY_SPALTEN_ANZAHL	; ist der GesamtSpaltenWert Grer als DISPLAY_SPALTEN
	 brlo	Code_Locate_T1		; auerhalb dann ABBRUCH,keine ZEICHENAUSGABE
	 rjmp	End_SerIn		;            ( rjmp weil sprungweite zu kurz)
	 
	 Code_Locate_T1:
	 or	DISPLAY_DAT, TEMP	; und Zeile zu SPlate in ein Byte f. Funktion
	 rcall	locateZ_S	 	; LOCATE Z_S
	 rjmp	End_SerIn		; und end
	; *** Befehl CLS ************
	Code_Clear_Display:
	 cpi	DISPLAY_DAT,ASC_CODE_CLEAR ; ist es das Code-Zeichen fr CLEAR_DISPLAY
	 brne	Code_LED_Steuer 	; nein, dann auf nchsten Befehlswort testen
	 ldi	DISPLAY_DAT,DISPLAY_CLEAR  ; sonst Clear Display	
	 rcall	Disp_Send_Code
	 clr	DISPLAY_DAT		; und auf position eins
	 rcall	locateZ_S
	 rjmp	End_SerIn		; und ENDE ohne Zeichenausgabe	
	; *** Befehl um LEDs zu steuern *********** 
	Code_LED_Steuer:
	 cpi	DISPLAY_DAT,ASC_CODE_LED ; ist es das Code-Zeichen fr LED ON/OFF
	 brne	Code_Beleucht_Display	; nein, dann test auf nchsten Befehl
	 Code_LED_Nr:			; erwarte welche LED verndert werden soll  
	 tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ?
	 breq	Code_LED_Nr		; nein ? dann warte
	 rcall	Read_Puffer		; sonst lesen und entsprechend auswerten
	 subi	DISPLAY_DAT,48		; aus ASC Zahl machen
	 cpi	DISPLAY_DAT,3		; welche LED ? 0-2
	 brlo	Code_LED_w		; auerhalb der LEDs dann ABBRUCH,keine ZEICHENAUSGABE
	 rjmp	End_SerIn		;               (rjmp weil sprunglnge zu kurz)
	 Code_LED_w:
	 mov	CODE_MERK, DISPLAY_DAT  ; Merke die Nummer und weiter zu -> teste ob LED an/AUS
	 ldi	TEMP,8			; schiebe die LED-Nummer auf richtigen Port
         two_h:				; 2hochLED-Nummer
	 tst	CODE_MERK
	 breq	n_two_h
	 lsl	TEMP
	 dec	CODE_MERK
	 rjmp	two_h
	 n_two_h:
	 mov	CODE_MERK,TEMP
	 Code_LED_Zustand:		; und soll LED AN o. AUS sein
	 tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ?
	 breq	Code_LED_Zustand	; nein ? dann warte
	 rcall	Read_Puffer		; sonst lesen und entsprechend auswerten
	 cpi	DISPLAY_DAT, 48		; Null '0' ? dann weiter ob AN oder blinken
	 brne	Code_LED_Nr_ANorBlink
	 in	DISPLAY_DAT, PORTD_D	; Zustand ? CTS usw nicht verndern 
	 or	DISPLAY_DAT,CODE_MERK	; jew. bit setzen
	 out	PORTD_D,DISPLAY_DAT
	 com	TEMP			; entpsr. bit in Timer-REgister lschen
	 and	TIMER_AKTIV_BYTE, TEMP	; damit led blinken ausgeschalten wird
	 rjmp	End_SerIn	
	 Code_LED_Nr_ANorBlink:
  	 cpi	DISPLAY_DAT, 49		; '1' ? dann LED AN sonst blinken setzen
  	 brne	Code_LED_Nr_Blink
	 in	DISPLAY_DAT, PORTD_D	; Zustand ? CTS usw nicht verndern
	 com 	CODE_MERK		; Complement (neg.)
	 and 	DISPLAY_DAT,CODE_MERK	; UND                jew. bit lschen
	 out	PORTD_D,DISPLAY_DAT
	 com	TEMP			; entpsr. bit in Timer-REgister lschen
	 and	TIMER_AKTIV_BYTE, TEMP	; damit led blinken ausgeschalten wird
	 rjmp	End_SerIn		; end
	 Code_LED_Nr_Blink:
	 cpi	DISPLAY_DAT, 50		; '2' ? dann LED Blinken sonst ende
	 brne	End_SerIn 
	 or	TIMER_AKTIV_BYTE, TEMP	; entpsr. bit in Timer-REgister setzen damit LED blinkt
	 rjmp	End_SerIn		; abfrage in Timer0 IRQ
	 ; *** Befehl fr Beleuchtungssteuerung *****
	 ; *** manuelle Steuerung(~B0/1) oder Automatik nach LDR (~B2)
	Code_Beleucht_Display: 
	 cpi	DISPLAY_DAT,ASC_CODE_BEL ; ist es das Code-Zeichen f. Beleuchtung
	 brne	Main_Send_Data_MIT_LOC	; nein, dann end
	 Code_Beleucht_Z:		; hole den Zustand den Beleuchtung bekommen soll
	 tst	PUFFER_AKT_LAENG	; ist ein Zeichen im Puffer ?
	 breq	Code_Beleucht_Z		; nein ? dann warte
	 rcall	Read_Puffer		; sonst lesen und entsprechend auswerten
	 cpi	DISPLAY_DAT,48 		; ist es  '0'
	 brne 	Code_Beleucht_ONorAUTO	; nein, dann Beleuchtung AN oder AUTOmatik
 	 mov	WORK , TIMER_AKTIV_BYTE	 ; Timerfunktion bergehen ohne Timer abzuschalten
 	 andi	WORK, 0b11111110	 ; Bit 1 ausschalten (Beleuchtung)
 	 mov	TIMER_AKTIV_BYTE ,  WORK	
	 cbi	PORTC_D,1		; und Beleuchtung AUS
	 rjmp	End_SerIn		; und ENDE ohne Zeichenausgabe	
	 Code_Beleucht_ONorAUTO:	; Beleuchtung An oder Automatik
 	 cpi    DISPLAY_DAT,49		; ist es '1' fr Beleuchtung AN oder hher f. AUTOmatik ?
 	 brne   Code_Beleucht_AUTO	; nein dann Automatik einschalten sonst Beleuchtung an
 	 mov	WORK , TIMER_AKTIV_BYTE	 ; Timerfunktion bergehen ohne Timer abzuschalten
 	 andi	WORK, 0b11111110	 ; Bit 1 ausschalten (Beleuchtung)
 	 mov	TIMER_AKTIV_BYTE ,  WORK	
	 sbi	PORTC_D,1		; und Beleuchtung AN
	 rjmp	End_SerIn		; und ENDE ohne Zeichenausgabe	
	 Code_Beleucht_AUTO:		; Automatik einschalten
	 cpi    DISPLAY_DAT,50		; ist es '2' fr Beleuchtung AUTO 
	 brne   Code_Beleucht_Fail	; nein dann Ende ohne Funktion
	 mov	WORK , TIMER_AKTIV_BYTE	 ; Timerfunktion fr Beleuchtung aktivieren
 	 ori	WORK, 0b00000001	 ; Bit 1 anschalten (Beleuchtung)
 	 mov	TIMER_AKTIV_BYTE ,  WORK	
	 Code_Beleucht_Fail:
	 rjmp	End_SerIn		; und ENDE ohne Zeichenausgabe		 
	; **********************
  Main_Send_Data_MIT_LOC:	
 	; **********************
 	; ** berspringen der 'toten' Bereiche des Displays, 
 	; ** damit alle Zeichen umlaufend angezeigt werden falls kein Positionsbefehl 
 	; ** vom Benutzer kommt
 	; !!! standart wenn keine besonderen Befehle kommen !!!
	push	DISPLAY_DAT		; rette Zeichen
	mov	DISPLAY_DAT,C_MERK	; Positionier Cursor entsprechend akt. Cursorposition
	rcall	locateZ_S
	
	; erhhre die Cursor Position um 1, inklusive Autmatischen Zeilenwechsel 
	andi 	DISPLAY_DAT, 0b00111111    ; lsche Zeile raus
	inc  	DISPLAY_DAT                ; Spalte++	
	cpi  	DISPLAY_DAT, DISPLAY_SPALTEN_ANZAHL ; jetzt grer als Spalten des DISPLAYs, 
	brlo 	NO_INC_ZEILE                        ;  also mu Zeile erhht werden ? u. Spalte gelscht
					 ; ja, Zeile erhhen	
	mov  	DISPLAY_DAT, C_MERK      ; Spalte wird ja eh null		                         
	swap	DISPLAY_DAT		 ; verschiebe ZeilenPosition von Bit7/8 uf Bit 0/1
	lsr     DISPLAY_DAT              ; 
	lsr     DISPLAY_DAT              ; ist verschoben
	inc	DISPLAY_DAT              ; erhhe Zeile
	andi    DISPLAY_DAT, 0b00000011  ; bertrag raus
	cpi	DISPLAY_DAT, DISPLAY_ZEILEN_ANZAHL  ; zustlich um auch Zeilen Einstellbar zu machen
	brlo	NO_RESET_CURSOR          ; Zeile ber Konfigurierte Display-Zeilenanzahl ? dann auf POS-Null 
	clr	C_MERK                   ; ja, dann Reset Zeile,Spalte auf Null
	rjmp    CURSOR_POS_END           
	NO_RESET_CURSOR:                 ; Zeile im erlaubten Bereich
	swap	DISPLAY_DAT		 ; Zeile wieder in oberen Bits
	lsl     DISPLAY_DAT              ; 
	lsl     DISPLAY_DAT              ; ist verschoben
	mov	C_MERK, DISPLAY_DAT	 ; Zeilen nach C_MERK, Spalte ist da berlauf Zeile ja null
	rjmp	CURSOR_POS_END
	NO_INC_ZEILE:                    ; Zeile mu nicht erhht werden
	andi    DISPLAY_DAT, 0b00111111  ; Eventuellen bertrag lschen
	ldi	WORK, 0b11000000         ; lsche Spalten aus C_MERK raus (C_MERK <r16 !)  
	and	C_MERK, WORK      	 ;  (zum ersetzen der spalteNr in C_MERk mit akt. SpaltenNr)
	or      C_MERK, DISPLAY_DAT      ; adde neue Spaltennummer ins C_MERK
	CURSOR_POS_END:	
	
	pop	DISPLAY_DAT		; und Zeichen wieder nach Display_Dat
 	; **********************
  Main_Send_Data_Ohne_LOC:
  	; Cursor wird ber Befehle(LOCATE des Benutzers) positioniert
	rcall	Disp_Send_Data
  End_SerIn:
	clr	DISPLAY_DAT		; lschen und zurck
	rjmp    Start_SerIn
	
; **** End-Main ****************************************************************
; ******************************************************************************



; ******************************************************************************
; **** Includes ****************************************************************

; *** Unterprogramm Speicherzugriffe vereinfachen
.INCLUDE 	"DM2-memory.inc"

; *** Unterprogramm PAUSE klein und gross....
.INCLUDE 	"DM2-pause.inc"

; *** Die Timer-IRQ
.INCLUDE	"DM2-timer.inc"

; *** Unterprogramme Zeichenempfang (UART und Puffer)
.INCLUDE	"DM2-uart_and_puffer.inc"

; *** Unterprogramme zum Ansteuern des Displays (Hardware)
.INCLUDE	"DM2-display_steuer.inc"

; *** Unterprogramm AD-Wandlung durchgefhrt (vorerst nur fr Helligkeitsmessung)
.INCLUDE 	"DM2-AD_wandlung.inc"

; *** Unterprogramm Tasterbehandlung (sende Zeichen an Computer wenn Taste gedrckt)
.INCLUDE 	"DM2-taster_behandlung.inc"

; *** Unterprogramm Intro anzeigen
.INCLUDE 	"DM2-intro.inc"

; *** Unterprogramm SETUP-MENU
.INCLUDE 	"DM2-setup.inc"

; *** Unterprogramm SETUP-MENU
.INCLUDE 	"DM2-math.inc"


; **** End *********************************************************************
; ******************************************************************************
