; ******************************************************************************
; ****** Definitionen der Register, Variablen, EEPROM-Inhalte u.a. *************


;*** Portbelegungen der Schaltung*********************************************
;* Port	  -	Aufgabe	   -	Pins     - 	Richtung
;*
;* PD0 		RxD		Pin 2		IN
;* PD1		TxD		Pin 3		OUT	
;* PD2 		CTS		Pin 4		OUT
;* PD3 		LED 1(3)	Pin 5		OUT
;* PD4 		LED 2(2)	Pin 6		OUT
;* PD5 		LED 3(1)	Pin 11		OUT
;* PD6 		Taster 1	Pin 12		IN
;* PD7 		Taster 2	Pin 13		IN
;*
;* PC0		Display E	Pin 23		OUT
;* PC1		Display Bel.	Pin 24		OUT
;* PC2          Taster 3	Pin 25		IN
;* PC3          Helligkeitssens Pin 26		IN
;* PC4		Taster 4	Pin 27		IN
;* PC5          AD-Eingang	Pin 28		IN          
;*
;* PB0		Display D0	Pin 14		OUT/IN
;* PB1		Display D1	Pin 15		OUT/IN
;* PB2		Display D2	Pin 16		OUT/IN
;* PB3		Display D3	Pin 17		OUT/IN
;* PB4		Display R/S	Pin 18		OUT
;* PB5		Display R/W	Pin 19		OUT
;* 
;******************************************************************************


; ****************** Anmerkung Berechnung Position von Texten ****
; Codierung erfolgt mittels eines Registers 
; Bit 8/7 beschreiben die Zeile - Bit 6-0 die Spalte
; damit ergibt sich 
; Zeile 0 = 0
; Zeile 1 = 64
; Zeile 2 = 128
; Zeile 3 = 192
; Spalten von 0 - 127
; 
; um den Wert zu erhalten addiert man nun beide Werte...
; Zeile 2, Spalte 8  -> 128 + 9 ergibt als HexWert 0x89 
;*****************************************************************

; *******************  Portadressen
.EQU    PIND       = 0x10
.EQU	PORTD_DDR  = 0x11
.EQU	PORTD_D    = 0x12

.EQU    PINC       = 0x13
.EQU	PORTC_DDR  = 0x14
.EQU	PORTC_D    = 0x15

.EQU    PINB       = 0x16
.EQU	PORTB_DDR  = 0x17
.EQU	PORTB_D    = 0x18

; ***** I/O-Memory
.EQU	RAMEND		= 0xDF		; Adresse SRAM-End
.EQU    STATUS_FLAG     = 0x3F		; Status Register
.EQU	SP		= 0x3D		; Stackpointer 

.EQU    EEADRESS	= 0x1E		; EEPROM adressregister
.EQU	EEDAT		= 0x1D		; 	 Datenregister
.EQU	EECONTR		= 0x1C		;        controlregister

.EQU	UART_UDR   	= 0x0C		; UART   Datenregister
.EQU    UART_UCSRA 	= 0x0B		;	 Contr./Statusreg.A
.EQU	UART_UCSRB   	= 0x0A		;	 Contr./Statusreg.B
.EQU	UART_UBRR_L  	= 0x09		;	 Baudratenreg. Low
.EQU	UART_UBRR_H	= 0x03		;	 Baudratenreg. High

.EQU	ADMUX		= 0x07		; Kanalselectregister f. AD-Wandler
.EQU	ADCSR		= 0x06		; AD-Controllregister
.EQU	ADCH		= 0x05		; AD-Data High
.EQU	ADCL		= 0x04		; AD-Data Low

.EQU	TIMSK		= 0x39		; Timer/Counter Interupt Mask Register
.EQU	TIFR		= 0x38		; Timer/Counter Interupt Flag Register
.EQU	TCCR1A		= 0x2F		; Timer/Counter1 Controll Register A  16bit
.EQU	TCCR1B		= 0x2E		; Timer/Counter1 Controll Register B
.EQU	TCCR0		= 0x33		; Timer/Counter 0 Controll Register 8bit


; *************** Variablen
.EQU	Sensor_Kanal_Hellig = 3		; auf welchen Kanal liegt Helligkeitssensor
;* spter fest eingeben da ja nicht mehr verndert wird


.EQU	UART_CR		= 0b10011000	; UCSRB, IRQ erlaubt, Empfang/Senden erlaubt
.EQU	UART_CR_OFF   	= 0b00011000	; int off

; vordef. Teiler f. Baudrate bei 8MHZ 2400, 9600, 14400, 19200, 38400 Baud
; Teiler=(Mhz/Baudrate/16)-1
.EQU 	UART_TEILER_2400	= 207
.EQU 	UART_TEILER_9600	= 51
.EQU 	UART_TEILER_14400	= 34
.EQU 	UART_TEILER_19200	= 25
.EQU 	UART_TEILER_38400	= 12

.EQU	PUFFER_LAENGE_GESAMT  = 22	; Puffer auf 22 Zeichen 
.EQU	PUFFER_LAENGE_RESERVE = 2	; ReserveZeichen falls CTS zu lang braucht 2 
					; kann passieren wenn Display zu lang braucht zum 
					; zeichen ausgeben
.EQU	ASC_UG	= 32	; innerhalb der ASC-Spanne werden Zeichen empfangen und gepuffert 
.EQU	ASC_OG	= 126   ; von SPACE(32) bis ~(126)
.EQU	ASC_CODE = 0x7E	; 126d '~' Steuerzeichen, wird auch gepuffert 
			; -> gehe auf CODE-EMPFANG und werte nachfolg. Zeichen aus
.EQU    ASC_CODE_Clear  = 0x43		 ; 67d 'C' - fr display clear 
.EQU    ASC_CODE_LED    = 0x4C  	 ; 76d 'L' - LED Steuern 'L' an/aus/blinken
.EQU	ASC_CODE_BEL    = 0x42		 ; 66d 'B' - Beleuchtung an/aus/automatik
.EQU	ASC_CODE_LOCATE = 0x50  	 ; 80d 'P' - Cursor Position Z_S

.EQU	DDR_READ_DISPLAY   = 0b00110000  ; Datenrichtung vordef. zum Einlesen v Display
.EQU    DDR_WRITE_DISPLAY  = 0b00111111  ;                       zum Ausgeben auf Display
.EQU    DISPLAY_FUNK_SET   = 0b00101000	 ; fr init_Display_ hier kann Font,
.EQU    DISPLAY_DISP       = 0b00001100  ; Cusour usw. gendert werden, s.Datenblatt 
.EQU 	DISPLAY_CLEAR      = 0b00000001  ; Befehle f. Display ..
.EQU 	DISPLAY_EMODE      = 0b00000110
.EQU	DISPLAY_DDRAM_NULL = 0b10000000  ; DDRAM NullAdresse (also Zeile0)




.if	DISPLAY_TYPE == 1                ; >>>>>>>>>>----    
                                         ; 16x4 HD44780
.EQU	DISPLAY_SPALTEN_ANZAHL = 16	 ; Spalten des Displays (1-63)
.EQU	DISPLAY_ZEILEN_ANZAHL  = 4	 ; Zeilen des Display   (1-4)
; Anmerkung    DISPLAY_LINE_NULL  entspricht DISPLAY_DDRAM_NULL		       
.EQU    DISPLAY_LINE_EINS  = 0x40        ; Adressen der Zeile, siehe Datenblatt
.EQU    DISPLAY_LINE_ZWEI  = 0x10        ;
.EQU    DISPLAY_LINE_DREI  = 0x50        ;

.elif	DISPLAY_TYPE==2                  ; >>>>>>-------- 
                                         ; 20x4 HD44780
.EQU	DISPLAY_SPALTEN_ANZAHL = 20	 ; Spalten des Displays (1-63)
.EQU	DISPLAY_ZEILEN_ANZAHL  = 4	 ; Zeilen des Display   (1-4)
; Anmerkung    DISPLAY_LINE_NULL  entspricht DISPLAY_DDRAM_NULL		       
.EQU    DISPLAY_LINE_EINS  = 0x40        ; Adressen der Zeile, siehe Datenblatt
.EQU    DISPLAY_LINE_ZWEI  = 0x14        ;
.EQU    DISPLAY_LINE_DREI  = 0x54        ;

.elif	DISPLAY_TYPE==3                  ; >>>>>>--------
; todo 
.elif	DISPLAY_TYPE==4                  ; >>>>>>-------- 
; todo 

.endif                                   ; <<<<<<<<<<---- 



.EQU	Timer0_Set	   = 0b00000010  ; Timer0 IRQ an
.EQU	Timer_Clock_0	   = 0b00000101  ; Timerfrequenz einstellen hier CK/1024
.EQU	Timer1_Set	   = 0b10000000  ; Timer1 IRQ an
.EQU	Timer_Clock_1	   = 0b00000011  ; Timerfrequenz einstellen hier CK/64

.EQU	ENTRPELL_MAX_TIME = 13	; Maximalwert fr entprellzeit - anpassen an Quarz
.EQU	ENTRPELL_MIN_TIME = 4	; Minimalwert fr entprellzeit   


;***** Standartwerte falls EEPROM ein Fehler !! ANPASSEN !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
.EQU	AKT_UART_TEILER_STD		= 12		; Standart 38k4
.EQU	Timer_Aktiv_Byte_Def_STD	= 0b00000111
.EQU	TASTER_T_Counter_Time_STD	= 8		; Standartzeit 8


.EQU    POS_INTRO_MITTE         = 0x80                  ; Fr den IntroText die Position
                                                        ; Zeile 2 , Spalte 0    128 + 0 = 0x80
                                     


; SETUP-Bildernummern

.EQU	Menu_MAIN		= 0
.EQU	Menu_Haupt_1		= 1

 .EQU	Menu_UART_Haupt		= 2
  .EQU	Menu_UART_BAUD_1	= 21
  .EQU	Menu_UART_BAUD_2	= 22
  .EQU	Menu_UART_FLUSS		= 23

 .EQU	Menu_Taster_Haupt	= 3

 .EQU	Menu_Bel_Haupt		= 4
  .EQU	Menu_Bel_Modus		= 41
  .EQU	Menu_Bel_Schwelle	= 42

.EQU	Menu_Haupt_2		= 5

 .EQU	Menu_AD_Haupt		= 6

 .EQU	Menu_Info_Haupt		= 7


.if	DISPLAY_TYPE == 1                ; >>>>>>>>>>---- 					
.EQU	POS1		= 0x00	; Links oben  - positionen fr SETUP, an entsprechd. Displaygre anpassen
.EQU	POS2		= 0x0C	; Rechts oben
				; Zeile 0 , Spalte 12     0 + C = C
.EQU	POS3		= 0xC0	; Links unten
                                ; Zeile 3 , Spalte 0      192 + 0 = 0xC0  Bitmuster Z(11)S(000000)
.EQU	POS4		= 0xCC	; Rechts unten
                                ; Zeile 3 , Spalte 12     192 + 12 = 0xCC
.EQU	POS_M		= 0x44	; ca. Mitte
                                ; Zeile 1 , Spalte 4      64 + 4 = 0x44

.EQU	POS_WAHL	= 0x85	; fr die Wahlmenus (z.B. schaltwert beleuchtung einstellen)
                                ; Z2,SP 5   128 + 5 = 0x85
.EQU	POS_PLUS	= 0xC1	; die +  in den obig. Menues
                                ; Z3, SP1   192 + 1 = 0xC1
.EQU	POS_MINUS	= 0xCE	; und -
                                ; Z3, SP14  192 + E = 0xCE
.EQU	POS_AKT		= 0xC6	; der aktuelle Wert in Menus
                                ; Z3, SP6   192 + 6 = 0xC6
.elif	DISPLAY_TYPE==2                  ; >>>>>>--------
.EQU	POS1		= 0x00	; Links oben  - positionen fr SETUP, an entsprechd. Displaygre anpassen
.EQU	POS2		= 0x10	; Rechts oben
				; Zeile 0 , Spalte 16     0 + 16 = 10
.EQU	POS3		= 0xC0	; Links unten
                                ; Zeile 3 , Spalte 0      192 + 0 = 0xC0  Bitmuster Z(11)S(000000)
.EQU	POS4		= 0xD0	; Rechts unten
                                ; Zeile 3 , Spalte 16     192 + 16 = 0xD0
.EQU	POS_M		= 0x46	; ca. Mitte
                                ; Zeile 1 , Spalte 6      64 + 6 = 0x46

.EQU	POS_WAHL	= 0x85	; fr die Wahlmenus (z.B. schaltwert beleuchtung einstellen)
                                ; Z2,SP 5   128 + 5 = 0x85
.EQU	POS_PLUS	= 0xC1	; die +  in den obig. Menues
                                ; Z3, SP1   192 + 1 = 0xC1
.EQU	POS_MINUS	= 0xD2	; und -
                                ; Z3, SP14  192 + 18 = 0xD2
.EQU	POS_AKT		= 0xC8	; der aktuelle Wert in Menus
                                ; Z3, SP6   192 + 8 = 0xC8
.elif	DISPLAY_TYPE==3                  ; >>>>>>--------
; todo 
.elif	DISPLAY_TYPE==4                  ; >>>>>>-------- 
; todo 

.endif                                   ; <<<<<<<<<<---- 



; ******************* Register
.DEF	MEMORY		 = r0	; Lesen aus Programmspeicher
.DEF	SAVE_STATUS_FLAG = r1	; wenn INT ausgefhrt, hier das StatusRegister sichern
.DEF	PAUSE_LANG	 = r2	; fr Lange Pause
.DEF	PUFFER_POS	 = r3	; Position d. Pufferzeigers
.DEF	PUFFER_AKT_LAENG = r4	; akt. anzahl v. Zeichen im Puffer (wenn Null dann Leer)
.DEF	C_MERK		 = r5	; Cursor Position merken
.DEF    TASTER_PRESSED   = r6   ; welcher taster wurde gedrckt ? TimerIRQ speichert 
				; gedrckten Taster in diesem Register
				
				; ( besser in TIMER_T1_COUNTER umbenennen )
.DEF	TIMER_T1_COUNTER = r7   ; Nutzung zum entprellen der Taster
.DEF    TIMER_T2_COUNTER = r8   ; 
.DEF	TIMER_T3_COUNTER = r9   ; 
.DEF    TIMER_T4_COUNTER = r10  ; Erkl.  bei Variablen   TASTER_Tx_Counter_Time

	; !! spter die Register aufteilen da nur ein halbes bentigt wird !!
.DEF	TIMER_AKTIV_BYTE = r11	; nicht den kompletten Teiler lahm legen, 
				; sondern abspeichern fr was (Hellig, LED usw. )
				; dieser aktiv sein soll und dann in der Timer-Routine
				; berspringen
				; Bit 0 = Beleuchtung
				; Bit 1 = Taster
				; Bit 2 = LED (blinken)
				; Bit 3/4/5 = welche LED soll blinken - FEST !
				; Bit 6 fr AD-Wandler -> spter
				; Bit 8 frei
				; siehe auch bei EEPROM.
.DEF	TIMER_TAKT	= r12	; wird nach ca 0.5 sec um eins erhht; nicht lschen in Routinen !
.DEF	UART_FLUSS_REG	= r13	; hierin wird EEPROM-Stelle eingelesen..
				; spart Codezeilen und Bytes (untiger hufiger EEPROM zugriff)
.DEF	REG_HELLIGK_SCHWELLE_MIN = r14 ; siehe bei EEPROM


.DEF    WORK		 = r16	; allgem. verwend.
.DEF	WORK2		 = r17	
.DEF    WORK_UART	 = r18
.DEF	TEMP		 = r19
.DEF    TEMP_DISP	 = r20  ; temp f. untpr. DATEN an DISP
.DEF    TEMP_DISP_DS	 = r21  ; temp f. untpr. DATEN an DISP
.DEF	DISPLAY_DAT	 = r22	; bergabe eines Bytes an Disp_Sendxxx
				; und 2. f. LocateZ_S
.DEF    PAUSEWERT	 = r23	; die Lnge der Pause
.DEF    PAUSE1    	 = r24	; f. Schleife in PauseRoutine
.DEF	CODE_MERK	 = r25	; fr Code-Zeichen ermitteln, zw-Speicher
; *** SRAM -Register
.DEF	X_L		 = r26  ; AD   -  Hauptanwendungszweck
.DEF	X_H		 = r27
.DEF	Y_L		 = r28	; UART_RX_C,
.DEF	Y_H		 = r29	;		
.DEF	Z_L		 = r30	; READ_PUFFER,  und SETUP Untermenu merker
.DEF	Z_H		 = r31	; 		


; ******************************************************************************
; **** daten die ins SRAM kommen (der EmpfangsPuffer)
.DSEG
PUFFER:			.byte	PUFFER_LAENGE_GESAMT


; ******************************************************************************
; **** daten die ins EEPROM kommen (Meldung und Menueinstellungen )
; **** ACHTUNG :
; ****           Fr wichtige Einstellungen wird Prfsumme gebildet da
; ****		 EEPROM-Daten nicht stabil sind !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
.ESEG

.ORG	0
	;****
	AKT_UART_TEILER:		.DB     12
	; aktuell eingestellte Baudrate 
	; Standart 38400   
	; Teiler = (CK/ BAUD*16) - 1 TABELLE 1
	; **** Prfsummenbildung da EEPROM-Daten immermal verndert werden !!!!
.ORG	1
	AKT_UART_TEILER_Pruef1:		.DB	12 
.ORG	2
	AKT_UART_TEILER_Pruef2:		.DB	12

.ORG	3
	;****
	Timer_Aktiv_Byte_Def:		.DB	0b00000111
	; Voreinstellung welche Timerroutinen aktiv sein sollen
	; Bit 0 = Beleuchtung	 
	; Bit 1 = Taster	 
	; Bit 2 = LED (blinken)	 
	; Bit 3/4/5 = welche LED soll blinken - FEST !
	; Bit 6 fr AD-Wandler -> spter
	; Bit 7 frei
	; **** Prfsummenbildung da EEPROM-Daten immermal verndert werden !!!!
.ORG	4
	Timer_Aktiv_Byte_Def_Pruef1:	.DB	0b00000111
.ORG	5
	Timer_Aktiv_Byte_Def_Pruef2:	.DB	0b00000111

.ORG	6
	;****
	TASTER_T_Counter_Time: 		.DB 	8
	; Nutzung zum entprellen der Taster - Zeit anpassen an eigene Taster im Menu
	; **** Prfsummenbildung da EEPROM-Daten immermal verndert werden !!!!
.ORG	7
	TASTER_T_Counter_Time_Pruef1:	.DB	8
.ORG	8
	TASTER_T_Counter_Time_Pruef2:	.DB	8


.ORG	9

UART_FLUSS:		.DB	0
; Abspeichern der Flusskontrolle cts/Xon ? 
; 0 = CTS  (Standart)
; 1 = Software (XON/XOFF)

Bel_ONorOFF:		.DB	0
; hier steht ob Beleuchtung Standartmig AN=1 oder AUS=0 sein soll
; trifft aber nur zu, wenn AUTOMATIK AUSgeschalten ist


Helligk_Schwelle_Min:	.DB	120
; nachtrgliches Andern wann Beleuchtung in AUTOMODUS angehen soll (bei welcher Helligkeit)
; wird wg. Geschwindigkeit am Anfang in Register abgespeichert



; ******* MACROS ********************************************************** 

; ******* schiebt Speicherposition einer Textmarke im Programmmemory nach Z
.macro Text_to_Z
	ldi 	Z_H, high(@0 *2)	; High Teil nach X_H
	ldi	Z_L, low(@0 *2)		; Low Teil nach X_L
.endmacro

; *************** End Macros **********************************************